@@ -18,34 +18,72 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+from collections import (
+    OrderedDict,
+    defaultdict,
+    namedtuple,
+)
 import traceback
-from operator import itemgetter
-from PyQt5 import QtCore, QtNetwork
-from picard import config, log
-from picard.coverart import coverart
-from picard.metadata import (Metadata,
-                             register_album_metadata_processor,
-                             run_album_metadata_processors,
-                             run_track_metadata_processors)
-from picard.dataobj import DataObject
-from picard.file import File
-from picard.track import Track
-from picard.script import ScriptParser, enabled_tagger_scripts_texts
-from picard.ui.item import Item
-from picard.util import format_time, mbid_validate
-from picard.util.imagelist import update_metadata_images
-from picard.util.textencoding import asciipunct
+
+from PyQt5 import (
+    QtCore,
+    QtNetwork,
+)
+
+from picard import (
+    config,
+    log,
+)
 from picard.cluster import Cluster
 from picard.collection import add_release_to_user_collections
+from picard.const import VARIOUS_ARTISTS_ID
+from picard.dataobj import DataObject
+from picard.file import File
 from picard.mbjson import (
+    medium_to_metadata,
     release_group_to_metadata,
     release_to_metadata,
-    medium_to_metadata,
     track_to_metadata,
 )
-from picard.const import VARIOUS_ARTISTS_ID
+from picard.metadata import (
+    Metadata,
+    run_album_metadata_processors,
+    run_track_metadata_processors,
+)
+from picard.plugin import (
+    PluginFunctions,
+    PluginPriority,
+)
+from picard.script import (
+    ScriptError,
+    ScriptParser,
+    enabled_tagger_scripts_texts,
+)
+from picard.track import Track
+from picard.util import (
+    find_best_match,
+    format_time,
+    mbid_validate,
+)
+from picard.util.imagelist import (
+    add_metadata_images,
+    remove_metadata_images,
+    update_metadata_images,
+)
+from picard.util.textencoding import asciipunct
 
-register_album_metadata_processor(coverart)
+from picard.ui.item import Item
+
+
+def _create_artist_node_dict(source_node):
+    return {x['artist']['id']: x['artist'] for x in source_node['artist-credit']}
+
+
+def _copy_artist_nodes(source, target_node):
+    for credit in target_node['artist-credit']:
+        artist_node = source.get(credit['artist']['id'])
+        if artist_node:
+            credit['artist'] = artist_node
 
 
 class AlbumArtist(DataObject):
@@ -68,7 +106,9 @@ class Album(DataObject, Item):
         self._files = 0
         self._requests = 0
         self._tracks_loaded = False
-        self._discid = discid
+        self._discids = set()
+        if discid:
+            self._discids.add(discid)
         self._after_load_callbacks = []
         self.unmatched_files = Cluster(_("Unmatched Files"), special=True, related_album=self, hide_if_empty=True)
         self.errors = []
@@ -97,6 +137,20 @@ class Album(DataObject, Item):
         self._album_artists.append(album_artist)
         return album_artist
 
+    def add_discid(self, discid):
+        if not discid:
+            return
+        self._discids.add(discid)
+        for track in self.tracks:
+            medium_discids = track.metadata.getall('~musicbrainz_discids')
+            track_discids = list(self._discids.intersection(medium_discids))
+            if track_discids:
+                track.metadata['musicbrainz_discid'] = track_discids
+                track.update()
+                for file in track.linked_files:
+                    file.metadata['musicbrainz_discid'] = track_discids
+                    file.update()
+
     def get_album_artists(self):
         """Returns the list of album artists (as AlbumArtist objects)"""
         return self._album_artists
@@ -119,6 +173,14 @@ class Album(DataObject, Item):
                 self.tagger.albums[release_id] = self
                 self.id = release_id
 
+        # Make the release artist nodes available, since they may
+        # contain supplementary data (aliases, tags, genres, ratings)
+        # which aren't present in the release group, track, or
+        # recording artist nodes. We can copy them into those places
+        # wherever the IDs match, so that the data is shared and
+        # available for use in mbjson.py and external plugins.
+        self._release_artist_nodes = _create_artist_node_dict(release_node)
+
         # Get release metadata
         m = self._new_metadata
         m.length = 0
@@ -128,13 +190,11 @@ class Album(DataObject, Item):
         rg.loaded_albums.add(self.id)
         rg.refcount += 1
 
+        _copy_artist_nodes(self._release_artist_nodes, rg_node)
         release_group_to_metadata(rg_node, rg.metadata, rg)
         m.copy(rg.metadata)
         release_to_metadata(release_node, m, album=self)
 
-        if self._discid:
-            m['musicbrainz_discid'] = self._discid
-
         # Custom VA name
         if m['musicbrainz_albumartistid'] == VARIOUS_ARTISTS_ID:
             m['albumartistsort'] = m['albumartist'] = config.setting['va_name']
@@ -151,7 +211,7 @@ class Album(DataObject, Item):
         # Run album metadata plugins
         try:
             run_album_metadata_processors(self, m, release_node)
-        except:
+        except BaseException:
             self.error_append(traceback.format_exc())
 
         self._release_node = release_node
@@ -214,8 +274,9 @@ class Album(DataObject, Item):
 
         if not self._tracks_loaded:
             artists = set()
-            totalalbumtracks = 0
+            all_media = []
             absolutetracknumber = 0
+
             va = self._new_metadata['musicbrainz_albumartistid'] == VARIOUS_ARTISTS_ID
 
             djmix_ars = {}
@@ -227,10 +288,18 @@ class Album(DataObject, Item):
                 mm.copy(self._new_metadata)
                 medium_to_metadata(medium_node, mm)
                 discpregap = False
+                format = medium_node.get('format')
+                if format:
+                    all_media.append(format)
 
                 for dj in djmix_ars.get(mm["discnumber"], []):
                     mm.add("djmixer", dj)
 
+                if 'discs' in medium_node:
+                    discids = [disc.get('id') for disc in medium_node['discs']]
+                    mm['~musicbrainz_discids'] = discids
+                    mm['musicbrainz_discid'] = list(self._discids.intersection(discids))
+
                 if "pregap" in medium_node:
                     discpregap = True
                     absolutetracknumber += 1
@@ -250,19 +319,20 @@ class Album(DataObject, Item):
                         track = self._finalize_loading_track(track_node, mm, artists, va, absolutetracknumber, discpregap)
                         track.metadata['~datatrack'] = "1"
 
-            totalalbumtracks = string_(absolutetracknumber)
+            totalalbumtracks = absolutetracknumber
+            self._new_metadata['~totalalbumtracks'] = totalalbumtracks
+            # Generate a list of unique media, but keep order of first appearance
+            self._new_metadata['media'] = " / ".join(list(OrderedDict.fromkeys(all_media)))
 
             for track in self._new_tracks:
                 track.metadata["~totalalbumtracks"] = totalalbumtracks
                 if len(artists) > 1:
                     track.metadata["~multiartist"] = "1"
             del self._release_node
+            del self._release_artist_nodes
             self._tracks_loaded = True
 
         if not self._requests:
-            for track in self._new_tracks:
-                track.orig_metadata.copy(track.metadata)
-
             self.enable_update_metadata_images(False)
             # Prepare parser for user's script
             for s_name, s_text in enabled_tagger_scripts_texts():
@@ -271,13 +341,13 @@ class Album(DataObject, Item):
                     # Run tagger script for each track
                     try:
                         parser.eval(s_text, track.metadata)
-                    except:
+                    except ScriptError:
                         log.exception("Failed to run tagger script %s on track", s_name)
                     track.metadata.strip_whitespace()
                 # Run tagger script for the album itself
                 try:
                     parser.eval(s_text, self._new_metadata)
-                except:
+                except ScriptError:
                     log.exception("Failed to run tagger script %s on album", s_name)
                 self._new_metadata.strip_whitespace()
 
@@ -308,6 +378,15 @@ class Album(DataObject, Item):
             self._after_load_callbacks = []
 
     def _finalize_loading_track(self, track_node, metadata, artists, va, absolutetracknumber, discpregap):
+        # As noted in `_parse_release` above, the release artist nodes
+        # may contain supplementary data that isn't present in track
+        # artist nodes. Similarly, the track artists may contain
+        # information which the recording artists don't. Copy this
+        # information across to wherever the artist IDs match.
+        _copy_artist_nodes(self._release_artist_nodes, track_node)
+        _copy_artist_nodes(self._release_artist_nodes, track_node['recording'])
+        _copy_artist_nodes(_create_artist_node_dict(track_node), track_node['recording'])
+
         track = Track(track_node['recording']['id'], self)
         self._new_tracks.append(track)
 
@@ -315,20 +394,23 @@ class Album(DataObject, Item):
         tm = track.metadata
         tm.copy(metadata)
         track_to_metadata(track_node, track)
-        track.metadata["~absolutetracknumber"] = absolutetracknumber
+        tm["~absolutetracknumber"] = absolutetracknumber
+        track.orig_metadata.copy(tm)
         track._customize_metadata()
 
         self._new_metadata.length += tm.length
         artists.add(tm["artist"])
         if va:
             tm["compilation"] = "1"
+        else:
+            del tm["compilation"]
         if discpregap:
             tm["~discpregap"] = "1"
 
         # Run track metadata plugins
         try:
-            run_track_metadata_processors(self, tm, self._release_node, track_node)
-        except:
+            run_track_metadata_processors(self, tm, track_node, self._release_node)
+        except BaseException:
             self.error_append(traceback.format_exc())
 
         return track
@@ -345,27 +427,25 @@ class Album(DataObject, Item):
         self.status = _("[loading album information]")
         if self.release_group:
             self.release_group.loaded = False
-            self.release_group.folksonomy_tags.clear()
+            self.release_group.genres.clear()
         self.metadata.clear()
-        self.folksonomy_tags.clear()
+        self.genres.clear()
         self.update()
         self._new_metadata = Metadata()
         self._new_tracks = []
         self._requests = 1
         self.errors = []
         require_authentication = False
-        inc = ['release-groups', 'media', 'recordings', 'artist-credits',
+        inc = ['release-groups', 'media', 'discids', 'recordings', 'artist-credits',
                'artists', 'aliases', 'labels', 'isrcs', 'collections']
+        if self.tagger.webservice.oauth_manager.is_authorized():
+            require_authentication = True
+            inc += ['user-collections']
         if config.setting['release_ars'] or config.setting['track_ars']:
             inc += ['artist-rels', 'release-rels', 'url-rels', 'recording-rels', 'work-rels']
             if config.setting['track_ars']:
                 inc += ['recording-level-rels', 'work-level-rels']
-        if config.setting['folksonomy_tags']:
-            if config.setting['only_my_tags']:
-                require_authentication = True
-                inc += ['user-tags']
-            else:
-                inc += ['tags']
+        require_authentication = self.set_genre_inc_params(inc) or require_authentication
         if config.setting['enable_ratings']:
             require_authentication = True
             inc += ['user-ratings']
@@ -391,63 +471,73 @@ class Album(DataObject, Item):
     def _add_file(self, track, file):
         self._files += 1
         self.update(update_tracks=False)
+        add_metadata_images(self, [file])
         file.metadata_images_changed.connect(self.update_metadata_images)
-        self.update_metadata_images()
 
     def _remove_file(self, track, file):
         self._files -= 1
         self.update(update_tracks=False)
         file.metadata_images_changed.disconnect(self.update_metadata_images)
-        self.update_metadata_images()
+        remove_metadata_images(self, [file])
 
-    def match_files(self, files, use_recordingid=True):
+    def _match_files(self, files, recordingid=None, threshold=0):
         """Match files to tracks on this album, based on metadata similarity or recordingid."""
+        tracks_cache = defaultdict(lambda: None)
+
+        def build_tracks_cache():
+            for track in self.tracks:
+                tm_recordingid = track.orig_metadata['musicbrainz_recordingid']
+                tm_tracknumber = track.orig_metadata['tracknumber']
+                tm_discnumber = track.orig_metadata['discnumber']
+                for tup in (
+                    (tm_recordingid, tm_tracknumber, tm_discnumber),
+                    (tm_recordingid, tm_tracknumber),
+                    (tm_recordingid, )):
+                    tracks_cache[tup] = track
+
+        SimMatchAlbum = namedtuple('SimMatchAlbum', 'similarity track')
+
         for file in list(files):
             if file.state == File.REMOVED:
                 continue
-            matches = []
-            recordingid = file.metadata['musicbrainz_recordingid']
-            if use_recordingid and mbid_validate(recordingid):
-                matches = self._get_recordingid_matches(file, recordingid)
-            if not matches:
+            # if we have a recordingid to match against, use that in priority
+            recid = recordingid or file.metadata['musicbrainz_recordingid']
+            if recid and mbid_validate(recid):
+                if not tracks_cache:
+                    build_tracks_cache()
+                tracknumber = file.metadata['tracknumber']
+                discnumber = file.metadata['discnumber']
+                track = (tracks_cache[(recid, tracknumber, discnumber)]
+                         or tracks_cache[(recid, tracknumber)]
+                         or tracks_cache[(recid, )])
+                if track:
+                    yield (file, track)
+                    continue
+
+            # try to match by similarity
+            def candidates():
                 for track in self.tracks:
-                    sim = track.metadata.compare(file.orig_metadata)
-                    if sim >= config.setting['track_matching_threshold']:
-                        matches.append((sim, track))
-            if matches:
-                matches.sort(key=itemgetter(0), reverse=True)
-                file.move(matches[0][1])
+                    yield SimMatchAlbum(
+                        similarity=track.metadata.compare(file.orig_metadata),
+                        track=track
+                    )
+                    QtCore.QCoreApplication.processEvents()
+
+            no_match = SimMatchAlbum(similarity=-1, track=self.unmatched_files)
+            best_match = find_best_match(candidates, no_match)
+
+            if best_match.similarity < threshold:
+                yield (file, no_match.track)
             else:
-                file.move(self.unmatched_files)
+                yield (file, best_match.result.track)
 
-    def match_file(self, file, recordingid=None):
-        """Match the file on a track on this album, based on recordingid or metadata similarity."""
-        if file.state == File.REMOVED:
-            return
-        if recordingid is not None:
-            matches = self._get_recordingid_matches(file, recordingid)
-            if matches:
-                matches.sort(key=itemgetter(0), reverse=True)
-                file.move(matches[0][1])
-                return
-        self.match_files([file], use_recordingid=False)
-
-    def _get_recordingid_matches(self, file, recordingid):
-        matches = []
-        tracknumber = file.metadata['tracknumber']
-        discnumber = file.metadata['discnumber']
-        for track in self.tracks:
-            tm = track.metadata
-            if recordingid == tm['musicbrainz_recordingid']:
-                if tracknumber == tm['tracknumber']:
-                    if discnumber == tm['discnumber']:
-                        matches.append((4.0, track))
-                        break
-                    else:
-                        matches.append((3.0, track))
-                else:
-                    matches.append((2.0, track))
-        return matches
+    def match_files(self, files, recordingid=None):
+        """Match and move files to tracks on this album, based on metadata similarity or recordingid."""
+        moves = self._match_files(files,
+                                  recordingid=recordingid,
+                                  threshold=config.setting['track_matching_threshold'])
+        for file, target in moves:
+            file.move(target)
 
     def can_save(self):
         return self._files > 0
@@ -543,10 +633,10 @@ class Album(DataObject, Item):
                 number_of_images = len(metadata.images)
                 if getattr(metadata, 'has_common_images', True):
                     text += ngettext("; %i image", "; %i images",
-                                      number_of_images) % number_of_images
+                                     number_of_images) % number_of_images
                 else:
                     text += ngettext("; %i image not in all tracks", "; %i different images among tracks",
-                                      number_of_images) % number_of_images
+                                     number_of_images) % number_of_images
                 return text + ')'
             else:
                 return title
@@ -558,8 +648,12 @@ class Album(DataObject, Item):
                 return ''
         elif column == 'artist':
             return self.metadata['albumartist']
+        elif column == 'tracknumber':
+            return self.metadata['~totalalbumtracks']
+        elif column == 'discnumber':
+            return self.metadata['totaldiscs']
         else:
-            return ''
+            return self.metadata[column]
 
     def switch_release_version(self, mbid):
         if mbid == self.id:
@@ -605,9 +699,11 @@ class NatAlbum(Album):
 
     def update(self, update_tracks=True):
         self.enable_update_metadata_images(False)
+        old_album_title = self.metadata["album"]
         self.metadata["album"] = config.setting["nat_name"]
         for track in self.tracks:
-            track.metadata["album"] = self.metadata["album"]
+            if old_album_title == track.metadata["album"]:
+                track.metadata["album"] = self.metadata["album"]
             for file in track.linked_files:
                 track.update_file_metadata(file)
         self.enable_update_metadata_images(True)
@@ -621,3 +717,21 @@ class NatAlbum(Album):
 
     def can_browser_lookup(self):
         return False
+
+
+_album_post_removal_processors = PluginFunctions(label='album_post_removal_processors')
+
+
+def register_album_post_removal_processor(function, priority=PluginPriority.NORMAL):
+    """Registers an album-removed processor.
+    Args:
+        function: function to call after album removal, it will be passed the album object
+        priority: optional, PluginPriority.NORMAL by default
+    Returns:
+        None
+    """
+    _album_post_removal_processors.register(function.__module__, function, priority)
+
+
+def run_album_post_removal_processors(album_object):
+    _album_post_removal_processors.run(album_object)
