@@ -57,7 +57,8 @@ from picard.util import (
     tracknum_from_filename,
 )
 from picard.util.filenaming import make_short_filename
-from picard.util.scripttofilename import script_to_filename
+from picard.util.preservedtags import PreservedTags
+from picard.util.scripttofilename import script_to_filename_with_metadata
 from picard.util.tags import PRESERVED_TAGS
 
 from picard.ui.item import Item
@@ -85,9 +86,10 @@ class File(QtCore.QObject, Item):
         "album": 5,
         "length": 10,
         "totaltracks": 4,
-        "releasetype": 20,
+        "releasetype": 14,
         "releasecountry": 2,
         "format": 2,
+        "isvideo": 2,
     }
 
     class PreserveTimesStatError(Exception):
@@ -185,12 +187,11 @@ class File(QtCore.QObject, Item):
 
     def copy_metadata(self, metadata, preserve_deleted=True):
         acoustid = self.metadata["acoustid_id"]
-        preserve = config.setting["preserved_tags"].strip()
         saved_metadata = {}
 
-        for tag in re.split(r"\s*,\s*", preserve) + PRESERVED_TAGS:
-            values = self.orig_metadata.getall(tag)
-            if values:
+        preserved_tags = PreservedTags()
+        for tag, values in self.orig_metadata.rawitems():
+            if tag in preserved_tags or tag in PRESERVED_TAGS:
                 saved_metadata[tag] = values
         deleted_tags = self.metadata.deleted_tags
         self.metadata.copy(metadata)
@@ -198,7 +199,7 @@ class File(QtCore.QObject, Item):
             for tag in deleted_tags:
                 del self.metadata[tag]
         for tag, values in saved_metadata.items():
-            self.metadata.set(tag, values)
+            self.metadata[tag] = values
 
         if acoustid and "acoustid_id" not in metadata.deleted_tags:
             self.metadata["acoustid_id"] = acoustid
@@ -341,7 +342,7 @@ class File(QtCore.QObject, Item):
         """Save the metadata."""
         raise NotImplementedError
 
-    def _script_to_filename(self, naming_format, file_metadata, settings=None):
+    def _script_to_filename(self, naming_format, file_metadata, file_extension, settings=None):
         if settings is None:
             settings = config.setting
         metadata = Metadata()
@@ -350,7 +351,11 @@ class File(QtCore.QObject, Item):
         else:
             metadata.copy(self.orig_metadata)
             metadata.update(file_metadata)
-        return script_to_filename(naming_format, metadata, file=self, settings=settings)
+        (filename, new_metadata) = script_to_filename_with_metadata(
+            naming_format, metadata, file=self, settings=settings)
+        # NOTE: the script_to_filename strips the extension away
+        ext = new_metadata.get('~extension', file_extension)
+        return filename + '.' + ext.lstrip('.')
 
     def _fixed_splitext(self, filename):
         # In case the filename is blank and only has the extension
@@ -370,9 +375,7 @@ class File(QtCore.QObject, Item):
         # expand the naming format
         naming_format = settings['file_naming_format']
         if naming_format:
-            new_filename = self._script_to_filename(naming_format, metadata, settings)
-            # NOTE: the _script_to_filename strips the extension away
-            new_filename = new_filename + ext
+            new_filename = self._script_to_filename(naming_format, metadata, ext, settings)
             if not settings['rename_files']:
                 new_filename = os.path.join(os.path.dirname(new_filename), old_filename)
             if not settings['move_files']:
@@ -513,7 +516,7 @@ class File(QtCore.QObject, Item):
                 self.parent.remove_file(self)
             self.parent = parent
             self.parent.add_file(self)
-            self._acoustid_update()
+            self.acoustid_update()
 
     def _move(self, parent):
         if parent != self.parent:
@@ -521,15 +524,16 @@ class File(QtCore.QObject, Item):
             if self.parent:
                 self.parent.remove_file(self)
             self.parent = parent
-            self._acoustid_update()
+            self.acoustid_update()
 
-    def _acoustid_update(self):
+    def acoustid_update(self):
         recording_id = None
         if self.parent and hasattr(self.parent, 'orig_metadata'):
             recording_id = self.parent.orig_metadata['musicbrainz_recordingid']
-        if not recording_id:
-            recording_id = self.metadata['musicbrainz_recordingid']
+            if not recording_id:
+                recording_id = self.metadata['musicbrainz_recordingid']
         self.tagger.acoustidmanager.update(self, recording_id)
+        self.update_item()
 
     @classmethod
     def supports_tag(cls, name):
@@ -544,15 +548,17 @@ class File(QtCore.QObject, Item):
         names = set(new_metadata.keys())
         names.update(self.orig_metadata.keys())
         clear_existing_tags = config.setting["clear_existing_tags"]
+        ignored_tags = config.setting["compare_ignore_tags"]
         for name in names:
-            if not name.startswith('~') and self.supports_tag(name):
+            if (not name.startswith('~') and self.supports_tag(name)
+                and name not in ignored_tags):
                 new_values = new_metadata.getall(name)
                 if not (new_values or clear_existing_tags
                         or name in new_metadata.deleted_tags):
                     continue
                 orig_values = self.orig_metadata.getall(name)
                 if orig_values != new_values:
-                    self.similarity = self.orig_metadata.compare(new_metadata)
+                    self.similarity = self.orig_metadata.compare(new_metadata, ignored_tags)
                     if self.state == File.NORMAL:
                         self.state = File.CHANGED
                     break
@@ -670,8 +676,9 @@ class File(QtCore.QObject, Item):
                 statusbar(N_("No matching tracks above the threshold for file '%(filename)s'"))
             else:
                 statusbar(N_("File '%(filename)s' identified!"))
-                (track_id, release_group_id, release_id, node) = trackmatch
+                (track_id, release_group_id, release_id, acoustid, node) = trackmatch
                 if lookuptype == File.LOOKUP_ACOUSTID:
+                    self.metadata['acoustid_id'] = acoustid
                     self.tagger.acoustidmanager.add(self, track_id)
                 if release_group_id is not None:
                     releasegroup = self.tagger.get_release_group_by_id(release_group_id)
@@ -698,13 +705,14 @@ class File(QtCore.QObject, Item):
         else:
             track_id = best_match.result.track['id']
             release_group_id, release_id, node = None, None, None
+            acoustid = best_match.result.track.get('acoustid', None)
 
             if best_match.result.release:
                 release_group_id = best_match.result.releasegroup['id']
                 release_id = best_match.result.release['id']
             elif 'title' in best_match.result.track:
                 node = best_match.result.track
-            return (track_id, release_group_id, release_id, node)
+            return (track_id, release_group_id, release_id, acoustid, node)
 
     def lookup_metadata(self):
         """Try to identify the file using the existing metadata."""
@@ -740,7 +748,7 @@ class File(QtCore.QObject, Item):
 
     def clear_pending(self):
         if self.state == File.PENDING:
-            self.state = File.NORMAL
+            self.state = File.NORMAL if self.similarity == 1.0 else File.CHANGED
             self.update_item()
 
     def update_item(self):
