@@ -35,8 +35,10 @@ from PyQt5 import (
     QtGui,
     QtWidgets,
 )
+from PyQt5.QtDBus import QDBusConnection
 
 from picard import (
+    PICARD_APP_ID,
     PICARD_APP_NAME,
     PICARD_DESKTOP_NAME,
     PICARD_FANCY_VERSION_STR,
@@ -141,9 +143,7 @@ class Tagger(QtWidgets.QApplication):
         if not IS_MACOS and not IS_HAIKU:
             self.setStyle('Fusion')
 
-        # Set the WM_CLASS to 'MusicBrainz-Picard' so desktop environments
-        # can use it to look up the app
-        super().__init__(['MusicBrainz-Picard'] + unparsed_args)
+        super().__init__(sys.argv)
         self.__class__.__instance = self
         config._setup(self, picard_args.config_file)
 
@@ -229,13 +229,14 @@ class Tagger(QtWidgets.QApplication):
 
         self.webservice = WebService()
         self.mb_api = MBAPIHelper(self.webservice)
-        self.acoustid_api = AcoustIdAPIHelper(self.webservice)
 
         load_user_collections()
 
         # Initialize fingerprinting
-        self._acoustid = acoustid.AcoustIDClient()
+        acoustid_api = AcoustIdAPIHelper(self.webservice)
+        self._acoustid = acoustid.AcoustIDClient(acoustid_api)
         self._acoustid.init()
+        self.acoustidmanager = AcoustIDManager(acoustid_api)
 
         # Load plugins
         self.pluginmanager = PluginManager()
@@ -250,7 +251,6 @@ class Tagger(QtWidgets.QApplication):
                 os.makedirs(USER_PLUGIN_DIR)
             self.pluginmanager.load_plugins_from_directory(USER_PLUGIN_DIR)
 
-        self.acoustidmanager = AcoustIDManager()
         self.browser_integration = BrowserIntegration()
 
         self.files = {}
@@ -391,7 +391,11 @@ class Tagger(QtWidgets.QApplication):
         if isinstance(event, thread.ProxyToMainEvent):
             event.run()
         elif event.type() == QtCore.QEvent.FileOpen:
-            self.add_files([event.file()])
+            file = event.file()
+            if os.path.isdir(file):
+                self.add_directory(file)
+            else:
+                self.add_files([file])
             # We should just return True here, except that seems to
             # cause the event's sender to get a -9874 error, so
             # apparently there's some magic inside QFileOpenEvent...
@@ -443,9 +447,11 @@ class Tagger(QtWidgets.QApplication):
         if isinstance(target, (Track, Cluster)):
             for file in files:
                 file.move(target)
+                QtCore.QCoreApplication.processEvents()
         elif isinstance(target, File):
             for file in files:
                 file.move(target.parent)
+                QtCore.QCoreApplication.processEvents()
         elif isinstance(target, Album):
             self.move_files_to_album(files, album=target)
         elif isinstance(target, ClusterList):
@@ -456,7 +462,10 @@ class Tagger(QtWidgets.QApplication):
         ignoreregex = None
         pattern = config.setting['ignore_regex']
         if pattern:
-            ignoreregex = re.compile(pattern)
+            try:
+                ignoreregex = re.compile(pattern)
+            except re.error as e:
+                log.error("Failed evaluating regular expression for ignore_regex: %s", e)
         ignore_hidden = config.setting["ignore_hidden_files"]
         new_files = []
         for filename in filenames:
@@ -786,6 +795,20 @@ class Tagger(QtWidgets.QApplication):
             self._acoustid.analyze(file, partial(file._lookup_finished,
                                                  File.LOOKUP_ACOUSTID))
 
+    def generate_fingerprints(self, objs):
+        """Generate the fingerprints without matching the files."""
+        if not self.use_acoustid:
+            return
+        files = self.get_files_from_objects(objs)
+
+        def finished(file, result):
+            file.acoustid_update()
+            file.clear_pending()
+
+        for file in files:
+            file.set_pending()
+            self._acoustid.fingerprint(file, partial(finished, file))
+
     # =======================================================================
     #  Metadata-based lookups
     # =======================================================================
@@ -811,6 +834,7 @@ class Tagger(QtWidgets.QApplication):
             cluster = self.load_cluster(name, artist)
             for file in sorted(files, key=attrgetter('discnumber', 'tracknumber', 'base_filename')):
                 file.move(cluster)
+                QtCore.QCoreApplication.processEvents()
 
     def load_cluster(self, name, artist):
         for cluster in self.clusters:
@@ -874,6 +898,14 @@ def process_picard_args():
     parser = argparse.ArgumentParser(
         epilog="If one of the filenames begins with a hyphen, use -- to separate the options from the filenames."
     )
+    # Qt default arguments. Parse them so Picard does not interpret the
+    # arguments as file names to load.
+    parser.add_argument("-style", nargs=1, help=argparse.SUPPRESS)
+    parser.add_argument("-stylesheet", nargs=1, help=argparse.SUPPRESS)
+    # Same for default X arguments
+    parser.add_argument("-display", nargs=1, help=argparse.SUPPRESS)
+
+    # Picard specific arguments
     parser.add_argument("-c", "--config-file", action='store',
                         default=None,
                         help="location of the configuration file")
@@ -902,6 +934,9 @@ def main(localedir=None, autoupdate=True):
     QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps)
     QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling)
 
+    # Enable mnemonics on all platforms, even macOS
+    QtGui.qt_set_sequence_auto_mnemonic(True)
+
     signal.signal(signal.SIGINT, signal.SIG_DFL)
 
     picard_args, unparsed_args = process_picard_args()
@@ -910,6 +945,10 @@ def main(localedir=None, autoupdate=True):
     if picard_args.long_version:
         return longversion()
 
+    if not (IS_WIN or IS_MACOS or IS_HAIKU):
+        dbus = QDBusConnection.sessionBus()
+        dbus.registerService(PICARD_APP_ID)
+
     tagger = Tagger(picard_args, unparsed_args, localedir, autoupdate)
 
     # Initialize Qt default translations
