@@ -18,7 +18,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+from collections import defaultdict
 from functools import partial
+from itertools import filterfalse
+import re
 import traceback
 
 from PyQt5 import QtCore
@@ -33,6 +36,10 @@ from picard.const import (
     VARIOUS_ARTISTS_ID,
 )
 from picard.dataobj import DataObject
+from picard.file import (
+    run_file_post_addition_to_track_processors,
+    run_file_post_removal_from_track_processors,
+)
 from picard.mbjson import recording_to_metadata
 from picard.metadata import (
     Metadata,
@@ -44,6 +51,7 @@ from picard.script import (
     enabled_tagger_scripts_texts,
 )
 from picard.util.imagelist import (
+    ImageList,
     add_metadata_images,
     remove_metadata_images,
     update_metadata_images,
@@ -52,6 +60,7 @@ from picard.util.textencoding import asciipunct
 
 from picard.ui.item import Item
 
+
 _TRANSLATE_TAGS = {
     "hip hop": "Hip-Hop",
     "synth-pop": "Synthpop",
@@ -59,6 +68,51 @@ _TRANSLATE_TAGS = {
 }
 
 
+class TagGenreFilter:
+
+    def __init__(self, filters):
+        self.errors = dict()
+        self.match_regexes = defaultdict(list)
+        for lineno, line in enumerate(filters.splitlines()):
+            line = line.strip()
+            if line and line[0] in ('+', '-'):
+                _list = line[0]
+                remain = line[1:].strip()
+                if not remain:
+                    continue
+                if len(remain) > 2 and remain[0] == '/' and remain[-1] == '/':
+                    remain = remain[1:-1]
+                    try:
+                        regex_search = re.compile(remain, re.IGNORECASE)
+                    except Exception as e:
+                        log.error("Failed to compile regex /%s/: %s", remain, e)
+                        self.errors[lineno] = str(e)
+                        regex_search = None
+                else:
+                    # FIXME?: only support '*' (not '?' or '[abc]')
+                    # replace multiple '*' by one
+                    star = re.escape('*')
+                    remain = re.sub(star + '+', '*', remain)
+                    regex = '.*'.join([re.escape(x) for x in remain.split('*')])
+                    regex_search = re.compile('^' + regex + '$', re.IGNORECASE)
+                if regex_search:
+                    self.match_regexes[_list].append(regex_search)
+
+    def skip(self, tag):
+        if not self.match_regexes:
+            return False
+        for regex in self.match_regexes['+']:
+            if regex.search(tag):
+                return False
+        for regex in self.match_regexes['-']:
+            if regex.search(tag):
+                return True
+        return False
+
+    def filter(self, list_of_tags):
+        return list(filterfalse(self.skip, list_of_tags))
+
+
 class TrackArtist(DataObject):
     def __init__(self, ta_id):
         super().__init__(ta_id)
@@ -83,12 +137,17 @@ class Track(DataObject, Item):
 
     def add_file(self, file):
         if file not in self.linked_files:
+            track_will_expand = self.num_linked_files == 1
             self.linked_files.append(file)
             self.num_linked_files += 1
         self.update_file_metadata(file)
         add_metadata_images(self, [file])
         self.album._add_file(self, file)
         file.metadata_images_changed.connect(self.update_orig_metadata_images)
+        run_file_post_addition_to_track_processors(self, file)
+        if track_will_expand:
+            # Files get expanded, ensure the existing item renders correctly
+            self.linked_files[0].update_item()
 
     def update_file_metadata(self, file):
         if file not in self.linked_files:
@@ -107,6 +166,7 @@ class Track(DataObject, Item):
         file.metadata_images_changed.disconnect(self.update_orig_metadata_images)
         self.album._remove_file(self, file)
         remove_metadata_images(self, [file])
+        run_file_post_removal_from_track_processors(self, file)
         self.update()
 
     def update(self):
@@ -146,7 +206,10 @@ class Track(DataObject, Item):
         if column == 'title':
             prefix = "%s-" % m['discnumber'] if m['discnumber'] and m['totaldiscs'] != "1" else ""
             return "%s%s  %s" % (prefix, m['tracknumber'].zfill(2), m['title'])
-        return m[column]
+        elif column in m:
+            return m[column]
+        elif self.num_linked_files == 1:
+            return self.linked_files[0].column(column)
 
     def is_video(self):
         return self.metadata['~video'] == '1'
@@ -225,10 +288,10 @@ class Track(DataObject, Item):
         # And generate the genre metadata tag
         maxtags = config.setting['max_genres']
         minusage = config.setting['min_genre_usage']
-        ignore_genres = self._get_ignored_folksonomy_tags()
+        tag_filter = TagGenreFilter(config.setting['genres_filter'])
         genre = []
         for usage, name in taglist[:maxtags]:
-            if name.lower() in ignore_genres:
+            if tag_filter.skip(name):
                 continue
             if usage < minusage:
                 break
@@ -239,13 +302,6 @@ class Track(DataObject, Item):
             genre = [join_genres.join(genre)]
         self.metadata['genre'] = genre
 
-    def _get_ignored_folksonomy_tags(self):
-        tags = []
-        ignore_genres = config.setting['ignore_genres']
-        if ignore_genres:
-            tags = [s.strip().lower() for s in ignore_genres.split(',')]
-        return tags
-
     def update_orig_metadata_images(self):
         update_metadata_images(self)
 
@@ -254,9 +310,9 @@ class Track(DataObject, Item):
             file.keep_original_images()
         self.update_orig_metadata_images()
         if self.linked_files:
-            self.metadata.images = self.orig_metadata.images[:]
+            self.metadata.images = self.orig_metadata.images.copy()
         else:
-            self.metadata.images = []
+            self.metadata.images = ImageList()
         self.update()
 
 
@@ -280,7 +336,7 @@ class NonAlbumTrack(Track):
         return super().column(column)
 
     def load(self, priority=False, refresh=False):
-        self.metadata.copy(self.album.metadata)
+        self.metadata.copy(self.album.metadata, copy_images=False)
         self.status = _("[loading recording information]")
         self.error = None
         self.loaded = False
@@ -323,8 +379,9 @@ class NonAlbumTrack(Track):
     def _parse_recording(self, recording):
         m = self.metadata
         recording_to_metadata(recording, m, self)
+        self.orig_metadata.copy(m)
         self._customize_metadata()
-        run_track_metadata_processors(self.album, m, None, recording)
+        run_track_metadata_processors(self.album, m, recording)
         for s_name, s_text in enabled_tagger_scripts_texts():
             parser = ScriptParser()
             try:
