@@ -38,6 +38,7 @@ from picard import (
 )
 from picard.util import (
     icontheme,
+    natsort,
     restore_method,
     throttle,
 )
@@ -51,7 +52,7 @@ class ResultTable(QtWidgets.QTableWidget):
     def __init__(self, parent):
         super().__init__(parent)
         self.setSelectionMode(
-            QtWidgets.QAbstractItemView.SingleSelection)
+            QtWidgets.QAbstractItemView.ExtendedSelection)
         self.setSelectionBehavior(
             QtWidgets.QAbstractItemView.SelectRows)
         self.setEditTriggers(
@@ -61,9 +62,8 @@ class ResultTable(QtWidgets.QTableWidget):
             QtWidgets.QHeaderView.Stretch)
         self.horizontalHeader().setSectionResizeMode(
             QtWidgets.QHeaderView.Interactive)
-        # only emit scrolled signal once per second
 
-        @throttle(1000)
+        @throttle(1000)  # only emit scrolled signal once per second
         def emit_scrolled(x):
             parent.scrolled.emit()
         self.horizontalScrollBar().valueChanged.connect(emit_scrolled)
@@ -169,9 +169,6 @@ class SearchBox(QtWidgets.QWidget):
 Retry = namedtuple("Retry", ["function", "query"])
 
 
-BY_NUMBER, BY_DURATION = range(2)
-
-
 class SortableTableWidgetItem(QtWidgets.QTableWidgetItem):
 
     def __init__(self, sort_key):
@@ -182,15 +179,6 @@ class SortableTableWidgetItem(QtWidgets.QTableWidgetItem):
         return self.sort_key < other.sort_key
 
 
-def to_seconds(timestr):
-    if not timestr:
-        return 0
-    seconds = 0
-    for part in timestr.split(':'):
-        seconds = seconds * 60 + int(part)
-    return seconds
-
-
 class SearchDialog(PicardDialog):
 
     defaultsize = QtCore.QSize(720, 360)
@@ -230,21 +218,14 @@ class SearchDialog(PicardDialog):
     def colpos(self, colname):
         return self.__colkeys.index(colname)
 
-    def set_table_item(self, row, colname, obj, key, default="", sort=None):
+    def set_table_item(self, row, colname, obj, key, default="", sortkey=None):
         # QVariant remembers the original type of the data
         # matching comparison operator will be used when sorting
         # get() will return a string, force conversion if asked to
         value = obj.get(key, default)
-        if sort == BY_DURATION:
-            item = SortableTableWidgetItem(to_seconds(value))
-        elif sort == BY_NUMBER:
-            try:
-                sortkey = float(value)
-            except ValueError:
-                sortkey = 0.0
-            item = SortableTableWidgetItem(sortkey)
-        else:
-            item = QtWidgets.QTableWidgetItem()
+        if sortkey is None:
+            sortkey = natsort.natkey(value)
+        item = SortableTableWidgetItem(sortkey)
         item.setData(QtCore.Qt.DisplayRole, value)
         pos = self.colpos(colname)
         if pos == 0:
@@ -393,9 +374,11 @@ class SearchDialog(PicardDialog):
 
     def accept(self):
         if self.table:
-            idx = self.table.selectionModel().selectedRows()[0]
-            row = self.table.itemFromIndex(idx).data(QtCore.Qt.UserRole)
-            self.accept_event(row)
+            selected_rows = []
+            for idx in self.table.selectionModel().selectedRows():
+                row = self.table.itemFromIndex(idx).data(QtCore.Qt.UserRole)
+                selected_rows.append(row)
+            self.accept_event(selected_rows)
         super().accept()
 
     @restore_method
