@@ -163,9 +163,13 @@ def strip_non_alnum(string):  # noqa: E302
     return _re_non_alphanum.sub(" ", string).strip()
 
 
-_re_slashes = re.compile(r'[\\/]', re.UNICODE)
-def sanitize_filename(string, repl="_"):  # noqa: E302
-    return _re_slashes.sub(repl, string)
+def sanitize_filename(string, repl="_", win_compat=False):
+    string = string.replace(os.sep, repl)
+    if os.altsep:
+        string = string.replace(os.altsep, repl)
+    if win_compat and os.altsep != '\\':
+        string = string.replace('\\', repl)
+    return string
 
 
 def _reverse_sortname(sortname):
@@ -518,45 +522,6 @@ def restore_method(func):
     return func_wrapper
 
 
-def compare_version_tuples(version1, version2):
-    """Compare Versions
-
-    Compares two Picard version tuples to determine whether the second tuple
-    contains a higher version number than the first tuple.
-
-    Args:
-        version1: The first version tuple to compare.  This will be used as
-                  the base for the comparison.
-        version2: The version tuple to be compared to the base version.
-
-    Returns:
-        -1 if version2 is lower than version1
-        0 if version2 is the same as version1
-        1 if version2 is higher than version1
-
-    Raises:
-        none
-    """
-
-    # Create test copies that can be modified
-    test1 = list(version1)
-    test2 = list(version2)
-
-    # Set sort order for release type element
-    test1[3] = 1 if test1[3] == 'final' else 0
-    if test1[3]:
-        test1[4] = 0
-    test2[3] = 1 if test2[3] == 'final' else 0
-    if test2[3]:
-        test2[4] = 0
-
-    # Compare elements in order
-    for x in range(0, 5):
-        if test1[x] != test2[x]:
-            return 1 if test1[x] < test2[x] else -1
-    return 0
-
-
 def reconnect(signal, newhandler=None, oldhandler=None):
     """
     Reconnect an handler to a signal
